/*
	SerpEncrypTool - A tool that uses GnuPG in order to manage your passwords and similar things
	Copyright (C) 2004  Michael "Serpedon" Walz

	This file is part of SerpEncrypTool.

	SerpEncrypTool is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	SerpEncrypTool is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SerpEncrypTool; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

# include <assert.h>
# include <DriveFkts.hpp>
# include "Klasse.GetPassphrase.hpp"
# include "Defines.hpp"
# include "GnuPG.hpp"

bool Encrypt ( const String & f_plainText, const String & f_destinyFile,
	/* <-- */	String * fp_stderr, HWND f_errorWindow = NULL );
bool Decrypt ( const String & f_passphrase, const String & f_sourceFile,
	/* <-- */	String * fp_plainText, String * fp_stderr );

bool Encrypt ( const String & f_plainText, const String & f_destinyFile,
	/* <-- */	String * fp_stderr, HWND f_errorWindow /* = NULL */ )
{
	STARTUPINFO startupinfo;

	GetStartupInfo ( & startupinfo );
	startupinfo.dwFlags = STARTF_USESHOWWINDOW;
	startupinfo.wShowWindow = SW_HIDE;

	FILE * tempFile;
	String tempFileName;
	do
	{
		char buffer [ MAX_PATH + 1 ];
		GetTempPath ( MAX_PATH, buffer );
		String tempPath = buffer;
		GetTempFileName ( tempPath, TempFilePrefix, 0, buffer );
		tempFileName = buffer;
		tempFile = fopen ( tempFileName, "w" );
	} while ( tempFile == NULL );

	fputs ( f_plainText, tempFile );
	fclose ( tempFile );

	String commandLine = EncryptCmdLine;
	commandLine.Replace ( ReplPlain,			f_plainText );
	commandLine.Replace ( ReplGnuPGExe,		G_Global.gpgExe );
	commandLine.Replace ( ReplHome,			G_Global.homeDir );
	commandLine.Replace ( ReplComment,		G_Global.comment );
	commandLine.Replace ( ReplKeyID,			G_Global.keyID );
	commandLine.Replace ( ReplCipherFile,	f_destinyFile );
	commandLine.Replace ( ReplPlainFile,	tempFileName );
	if ( ! G_Global.noVersion )
		commandLine.Replace ( ReplNoVersion, "" );
	if ( ! G_Global.verbose )
		commandLine.Replace ( ReplVerbose, "" );

	String stdoutText;
	DWORD resultValue;
	PipeCommand ( commandLine, "", & stdoutText, fp_stderr, & resultValue );

	if ( ! DeleteFile ( tempFileName ) )
	{
		String message;
		message.Format ( TempFileNotDel, tempFileName );
		MessageBox ( f_errorWindow, message, TempFileNotDelTitel, MB_OK | MB_ICONEXCLAMATION );
	}

	return resultValue == 0;
}

bool Decrypt ( const String & f_passphrase, const String & f_sourceFile,
	/* <-- */	String * fp_plainText, String * fp_stderr )
{
	String commandLine = DecryptCmdLine;
	commandLine.Replace ( ReplGnuPGExe,		G_Global.gpgExe );
	commandLine.Replace ( ReplHome,			G_Global.homeDir );
	commandLine.Replace ( ReplCipherFile,	f_sourceFile );
	if ( ! G_Global.verbose )
		commandLine.Replace ( ReplVerbose, "" );

	DWORD result;  
	PipeCommand ( commandLine, f_passphrase + "\r\n", fp_plainText, fp_stderr, & result );
	
	return result == 0;
}

bool EncryptTotal ( const String & f_plainText, const String & f_destinyFile,
	/* <-- */		  HWND f_errorWindow /* = NULL */ )
{
	String message;
	bool result = Encrypt ( f_plainText, f_destinyFile, & message, f_errorWindow );

	switch ( G_Global.outputType )
	{
		case OUTPUT_ON_ERROR:
			if ( result )
				break;
		case OUTPUT_ALWAYS:
			MessageBox ( f_errorWindow, message, result ? EncryptionSuccess: EncryptionFailed,
				/* <-- */		MB_OK | ( result ? MB_ICONINFORMATION : MB_ICONSTOP ) );
		case OUTPUT_NEVER:
			break;
		default:
			assert ( false );
	}
	return result;
}

bool DecryptTotal ( const String & f_sourceFile, String * fp_plainText,
	/* <-- */		  HWND f_errorWindow /* = NULL */ )
{
	
	c_GetPassphrase getPassphrase;

	getPassphrase.Init ( f_errorWindow );
	if ( getPassphrase.DoModal ( ) != IDOK )
	{
		if ( G_Global.outputType != OUTPUT_NEVER )
			MessageBox ( f_errorWindow, BadPassphrase, DecryptionFailed, MB_OK | MB_ICONSTOP );

		return false;
	}

	String message;
	bool result = Decrypt ( getPassphrase.GetPassphrase ( ), f_sourceFile, fp_plainText, & message );

	switch ( G_Global.outputType )
	{
		case OUTPUT_ON_ERROR:
			if ( result )
				break;
		case OUTPUT_ALWAYS:
			MessageBox ( f_errorWindow, message, result ? DecryptionSuccess: DecryptionFailed,
				/* <-- */		MB_OK | ( result ? MB_ICONINFORMATION : MB_ICONSTOP ) );
		case OUTPUT_NEVER:
			break;
		default:
			assert ( false );
	}

	while ( fp_plainText->Replace ( "\r\n", "\n" ) )
		; // Leerzeile

	return result;
}
