/*
	SerpEncrypTool - A tool that uses GnuPG in order to manage your passwords and similar things
	Copyright (C) 2004  Michael "Serpedon" Walz

	This file is part of SerpEncrypTool.

	SerpEncrypTool is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	SerpEncrypTool is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SerpEncrypTool; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

# include <assert.h>
# include <DriveFkts.hpp>
# include "Defines.hpp"
# include "Ressource.h"
# include "Klasse.SubDialog.hpp"
# include "Klasse.NewEntry.hpp"
# include "Klasse.MainDialog.hpp"
# include "WindowFkts.hpp"
# include "GnuPG.hpp"

class c_MainDialogEntry
{
public:
	String name;
	String filename;
	String CreateEntry ( )
	{
		String result;
		result.Format ( SubDialogEntryFormat, name, filename );
		return result;
	}
	String CreateDataFromEntry ( )
	{
		String result;
		result.Format ( "%s=%s\n", name, filename );
		return result;
	}
	bool Init ( const String & f_data )
	{
		int pos = f_data.Find ( "=" );
		
		// Erste Stelle wrde heien, kein Name vorhanden
		// -1 heit kein = drin
		if ( pos <= 0 )
			return false;

		name = f_data.Mid ( 0, pos );

		filename = f_data.Mid ( pos + 1 );

		if ( name.Length ( ) == 0 || filename.Length ( ) == 0 )
			return false;

		return true;
	}
};

c_MainDialog::c_MainDialog ( ) :
	m_parent ( NULL ), m_hWnd ( NULL )
{
	# if _DEBUG
		m_isInit = false;
		m_wasModal = false;
	# endif
}

c_MainDialog::~c_MainDialog ( )
{
}


bool c_MainDialog::Init ( HWND f_parent, const String & f_data )
{
	# if _DEBUG
		assert ( ! m_isInit );
		m_isInit = true;
	# endif
	
	m_parent	= f_parent;
	m_data	= f_data;

	if ( IsWindow ( m_parent ) )
		return true;

	return false;
}

int c_MainDialog::DoModal ( )
{
	# if _DEBUG
		assert ( m_isInit );
		assert ( ! m_wasModal );
		m_wasModal = true;
	# endif
	return DialogBoxParam ( G_hInstance,
		/* <-- */				MAKEINTRESOURCE ( IDD_MAIN_DIALOG ),
		/* <-- */				m_parent,
		/* <-- */				DialogProc,
		/* <-- */				( LPARAM ) this );
}

/* static */ BOOL __stdcall c_MainDialog::DialogProc ( 
	/* <-- */				HWND f_hWnd, UINT f_message, WPARAM f_wParam, LPARAM f_lParam )
{
	c_MainDialog * pThis =
		/* <-- */ ( c_MainDialog * ) GetWindowLong ( f_hWnd, DWL_USER );

	switch ( f_message )
	{
	case WM_INITDIALOG:
		pThis = ( c_MainDialog * ) f_lParam;
		SetWindowLong ( f_hWnd, DWL_USER, ( long ) pThis );
		if ( ! pThis->InitDialog ( f_hWnd ) )
			SendMessage ( f_hWnd, WM_CLOSE, 0, 0 );
		return true;

	case WM_COMMAND:
		switch ( LOWORD ( f_wParam ) )
		{
		case IDOK:
			if ( G_Global.disableEnter )
				break;
		case ID_OPEN_ENTRY:
			pThis->OpenEntry ( );
			return true;
		case IDCANCEL:
			SendMessage ( f_hWnd, WM_CLOSE, 0, 0 );
			return true;
		case ID_SHOW_ENTRY:
			pThis->ShowEntry ( );
			return true;
		case ID_ADD_ENTRY:
			pThis->AddEntry ( );
			return true;
		case ID_DELETE_ENTRY:
			pThis->DeleteEntry ( );
			return true;
		case ID_EDIT_ENTRY:
			pThis->EditEntry ( );
			return true;
		case IDC_LIST_DATA:
			if ( HIWORD ( f_wParam ) == LBN_DBLCLK )
			{
				if ( ! G_Global.disableDoubleClick )
					pThis->OpenEntry ( );
				return true;
			}
			break;
		}
		break;
	case WM_CLOSE:
		return pThis->CloseDialog ( );
	}

	return false;
}


bool c_MainDialog::InitDialog ( HWND f_hWnd )
{
	m_hWnd = f_hWnd;

	// Icon fr das Fenster setzen
	SendMessage ( m_hWnd, WM_SETICON, ICON_BIG,
		/* <-- */	( LPARAM ) LoadIcon ( G_hInstance, MAKEINTRESOURCE ( IDI_ICON ) ) );

	String temp = m_data;

	const char * seperator = "\n";
	char * token = strtok ( temp, seperator );
   while ( token != NULL )
   {
		c_MainDialogEntry * p_newEntry = new c_MainDialogEntry ( );
		if ( p_newEntry == NULL )
			return false;

		if ( ! p_newEntry->Init ( token ) )
		{
			MessageBox ( m_hWnd, InvalidToken, InvalidTokenTitel, MB_OK | MB_ICONEXCLAMATION );
			delete p_newEntry;
			return false;
		}

		AddListBoxItemAndLink ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ), p_newEntry->CreateEntry ( ), p_newEntry );

      token = strtok ( NULL, seperator );
   }

	return true;
}

bool c_MainDialog::CloseDialog ( )
{
	// Neue Daten aus dem Listenobjekt auslesen
	int amount = SendMessage ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ), LB_GETCOUNT, 0, 0 );
	if ( amount != LB_ERR )
	{
		m_data = "";
		for ( int count = 0; count < amount; count++ )
		{
			c_MainDialogEntry * p_entry = ( c_MainDialogEntry * )
				/* <-- */							SendMessage ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ),
				/* <-- */							LB_GETITEMDATA, count, 0 );

			m_data += p_entry->CreateDataFromEntry ( );
			
			delete p_entry;
		}
	}
	else
		MessageBox ( m_hWnd, ErrorWhileSavingEntries, SavingFailed, MB_OK | MB_ICONEXCLAMATION );

	EndDialog ( m_hWnd, 0 );
	return true;
}

String c_MainDialog::GetNewData ( )
{
	assert ( m_wasModal );
	return m_data;
}

bool c_MainDialog::OpenEntry ( )
{
	c_MainDialogEntry * p_entry = ( c_MainDialogEntry * ) GetCurrentSelection ( NULL );

	if ( p_entry )
	{		
		String data;
		if ( ExistFile ( p_entry->filename ) )
		{
			if ( ! DecryptTotal ( p_entry->filename, & data, m_hWnd ) )
				return false;
		}

		c_SubDialog subDialog;
		subDialog.Init ( m_hWnd, data );

		subDialog.DoModal ( );

		String newData = subDialog.GetNewData ( );

		if ( newData != data )
			EncryptTotal ( newData, p_entry->filename, m_hWnd );
	}
	else
		MessageBox ( m_hWnd, NoEntrySelected, NoEntrySelectedTitel, MB_OK | MB_ICONEXCLAMATION );

	return true;
}


bool c_MainDialog::ShowEntry ( )
{
	c_MainDialogEntry * p_entry = ( c_MainDialogEntry * ) GetCurrentSelection ( NULL );

	if ( p_entry )
		MessageBox ( m_hWnd, p_entry->filename, p_entry->name, MB_OK | MB_ICONINFORMATION );
	else
		MessageBox ( m_hWnd, NoEntrySelected, NoEntrySelectedTitel, MB_OK | MB_ICONEXCLAMATION );

	return true;
}

bool c_MainDialog::EditEntry ( )
{
	int index;
	c_MainDialogEntry * p_entry = ( c_MainDialogEntry * ) GetCurrentSelection ( & index );

	bool result = false;
	
	if ( p_entry )
	{
		c_NewEntry newEntry;
		if ( newEntry.Init ( m_hWnd, p_entry->name, p_entry->filename ) )
		{
			if ( newEntry.DoModal ( ) == IDOK )
			{
				MoveFile ( p_entry->filename, newEntry.GetFileName ( ) );
				SendMessage ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ), LB_DELETESTRING, index, 0 );

				p_entry->filename	= newEntry.GetFileName ( );
				p_entry->name		= newEntry.GetEntryName ( );

				if ( AddListBoxItemAndLink ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ),
					/* <-- */		p_entry->CreateEntry ( ), p_entry ) != LB_ERR )
					result = true;
			}
		}
	}
	else
		MessageBox ( m_hWnd, NoEntrySelected, NoEntrySelectedTitel, MB_OK | MB_ICONEXCLAMATION );

	return result;
}

bool c_MainDialog::AddEntry ( )
{
	bool result = false;
	c_NewEntry newEntry;
	if ( newEntry.Init ( m_hWnd ) )
	{
		if ( newEntry.DoModal ( ) == IDOK )
		{
			c_MainDialogEntry * p_newEntry = new c_MainDialogEntry ( );
			if ( p_newEntry )
			{
				p_newEntry->filename	= newEntry.GetFileName ( );
				p_newEntry->name		= newEntry.GetEntryName ( );

				bool toBeContinued = true;

				if ( ExistFile ( p_newEntry->filename ) )
				{
					String message;
					message.Format ( FileExistAllreadyFormat, p_newEntry->filename );
					if ( IDYES != MessageBox ( m_hWnd, message, FileExistAllready,
						/* <-- */					MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2 ) )
					{
						toBeContinued = false;
						delete p_newEntry;
					}
				}

				if ( toBeContinued )
				{

					FILE * file = fopen ( p_newEntry->filename, "w" );
					if ( file == NULL )
					{
						String message;
						message.Format ( FileCouldNotBeCreated, p_newEntry->filename );
						MessageBox ( m_hWnd, message, FileNotCreatedTitel, MB_OK | MB_ICONEXCLAMATION );
						delete p_newEntry;
					}
					else
					{
						fclose ( file );
						DeleteFile ( p_newEntry->filename );

						if ( AddListBoxItemAndLink ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ),
							/* <-- */		p_newEntry->CreateEntry ( ), p_newEntry ) != LB_ERR )
							result = true;
					}
				}
			}
		}
	}

	return result;
}

bool c_MainDialog::DeleteEntry ( )
{
	int index;
	c_MainDialogEntry * p_entry = ( c_MainDialogEntry * ) GetCurrentSelection ( & index );

	if ( p_entry )
	{
		if ( IDYES == MessageBox ( m_hWnd, ReallyDelete, RealleyDeleteTitel,
			/* <-- */					MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2 ) )
		{
			SendMessage ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ), LB_DELETESTRING, index, 0 );
			if ( ! DeleteFile ( p_entry->filename ) )
			{	
				String message;
				message.Format ( FileNotDeletedFormat, p_entry->filename );
				MessageBox ( m_hWnd, message, FileNotDeletedTitel, MB_OK | MB_ICONEXCLAMATION );
			}

			delete p_entry;
		}
	}
	else
		MessageBox ( m_hWnd, NoEntrySelected, NoEntrySelectedTitel, MB_OK | MB_ICONEXCLAMATION );

	return true;
}

void * c_MainDialog::GetCurrentSelection ( int * fp_curSel )
{
	int temp;
	if ( fp_curSel == NULL )
		fp_curSel = & temp;

	( * fp_curSel ) = SendMessage ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ), 
		/* <-- */						  LB_GETCURSEL, ( * fp_curSel ), 0 ); 

	void * result = ( void * ) SendMessage ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ), 
		/* <-- */									  LB_GETITEMDATA, ( * fp_curSel ), 0 );

	if ( result == ( void * ) LB_ERR )
		return NULL;

	return result;
}