/*
	SerpEncrypTool - A tool that uses GnuPG in order to manage your passwords and similar things
	Copyright (C) 2004  Michael "Serpedon" Walz

	This file is part of SerpEncrypTool.

	SerpEncrypTool is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	SerpEncrypTool is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SerpEncrypTool; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

# include <assert.h>
# include "Defines.hpp"
# include "Ressource.h"
# include "Klasse.NewEntry.hpp"
# include "WindowFkts.hpp"

c_NewEntry::c_NewEntry ( ) :
	m_parent ( NULL ), m_hWnd ( NULL ), result ( IDCANCEL )
{
	# if _DEBUG
		m_isInit = false;
		m_wasModal = false;
	# endif
}

c_NewEntry::~c_NewEntry ( )
{
}


bool c_NewEntry::Init (	HWND f_parent, const String & f_entryName /* = "" */,
	/* <-- */				const String & f_fileName /* = "" */ )
{
	# if _DEBUG
		assert ( ! m_isInit );
		m_isInit = true;
	# endif
	
	m_parent		= f_parent;
	m_entryName	= f_entryName;
	m_fileName	= f_fileName;

	if ( IsWindow ( m_parent ) )
		return true;

	return false;
}

int c_NewEntry::DoModal ( )
{
	# if _DEBUG
		assert ( ! m_wasModal );
		assert ( m_isInit );
		m_wasModal = true;
	# endif
	return DialogBoxParam ( G_hInstance,
		/* <-- */				MAKEINTRESOURCE ( IDD_NEW_ENTRY ),
		/* <-- */				m_parent,
		/* <-- */				DialogProc,
		/* <-- */				( LPARAM ) this );
}

/* static */ BOOL __stdcall c_NewEntry::DialogProc ( 
	/* <-- */				HWND f_hWnd, UINT f_message, WPARAM f_wParam, LPARAM f_lParam )
{
	c_NewEntry * pThis =
		/* <-- */ ( c_NewEntry * ) GetWindowLong ( f_hWnd, DWL_USER );

	switch ( f_message )
	{
	case WM_INITDIALOG:
		pThis = ( c_NewEntry * ) f_lParam;
		SetWindowLong ( f_hWnd, DWL_USER, ( long ) pThis );

		pThis->InitDialog ( f_hWnd );
			
		return true;

	case WM_COMMAND:
		switch ( LOWORD ( f_wParam ) )
		{
		case IDOK:
			if ( ! pThis->SaveData ( ) )
				return true;
		case IDCANCEL:
			SendMessage ( f_hWnd, WM_CLOSE, 0, 0 );
			return true;
		}
		break;

	case WM_CLOSE:
		EndDialog ( f_hWnd, pThis->result );
		return true;
	}

	return false;
}


bool c_NewEntry::InitDialog ( HWND f_hWnd )
{
	m_hWnd = f_hWnd;

	// Icon fr das Fenster setzen
	SendMessage ( m_hWnd, WM_SETICON, ICON_BIG,
		/* <-- */	( LPARAM ) LoadIcon ( G_hInstance, MAKEINTRESOURCE ( IDI_ICON ) ) );

	SendMessage ( GetDlgItem ( m_hWnd, IDC_ENTRY_NAME	), WM_SETTEXT, 0, ( LPARAM ) ( LPCTSTR ) m_entryName );
	SendMessage ( GetDlgItem ( m_hWnd, IDC_FILE_NAME	), WM_SETTEXT, 0, ( LPARAM ) ( LPCTSTR ) m_fileName );

	return true;
}


String c_NewEntry::GetEntryName ( )
{
	assert ( m_wasModal );
	return m_entryName;
}

String c_NewEntry::GetFileName ( )
{
	assert ( m_wasModal );
	return m_fileName;
}

bool c_NewEntry::SaveData ( )
{
	m_entryName	= GetEditText ( GetDlgItem ( m_hWnd, IDC_ENTRY_NAME ) );

	if ( m_entryName.Find ( "*" ) != -1 || m_entryName.Find ( "=" ) != -1 )
	{
		MessageBox ( m_hWnd, InvalidChars, InvalidCharsTitel, MB_OK | MB_ICONEXCLAMATION );
		return false;
	}

	m_fileName	= GetEditText ( GetDlgItem ( m_hWnd, IDC_FILE_NAME ) );

	if ( m_entryName.Length ( ) == 0 || m_fileName.Length ( ) == 0 )
	{
		MessageBox ( m_hWnd, InvalidLength, InvalidLengthTitel, MB_OK | MB_ICONEXCLAMATION );
		return false;
	}

	result = IDOK;

	return true;
}
