/*
	SerpEncrypTool - A tool that uses GnuPG in order to manage your passwords and similar things
	Copyright (C) 2004  Michael "Serpedon" Walz

	This file is part of SerpEncrypTool.

	SerpEncrypTool is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	SerpEncrypTool is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SerpEncrypTool; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

# include <assert.h>
# include "Defines.hpp"
# include "Ressource.h"
# include "Klasse.NewSubEntry.hpp"
# include "WindowFkts.hpp"

c_NewSubEntry::c_NewSubEntry ( ) :
	m_parent ( NULL ), m_hWnd ( NULL ), result ( IDCANCEL )
{
	# if _DEBUG
		m_isInit = false;
		m_wasModal = false;
	# endif
}

c_NewSubEntry::~c_NewSubEntry ( )
{
}


bool c_NewSubEntry::Init (	HWND f_parent, const String & f_entryName /* = "" */,
	/* <-- */	const String & f_value /* = ""*/, bool f_hideValue /*= false */ )
{
	# if _DEBUG
		assert ( ! m_isInit );
		m_isInit = true;
	# endif
	
	m_parent = f_parent;

	m_entryName	= f_entryName;
	m_value		= f_value;
	m_hideValue	= f_hideValue;

	if ( IsWindow ( m_parent ) )
		return true;

	return false;
}

int c_NewSubEntry::DoModal ( )
{
	# if _DEBUG
		assert ( ! m_wasModal );
		assert ( m_isInit );
		m_wasModal = true;
	# endif
	return DialogBoxParam ( G_hInstance,
		/* <-- */				MAKEINTRESOURCE ( IDD_NEW_SUB_ENTRY ),
		/* <-- */				m_parent,
		/* <-- */				DialogProc,
		/* <-- */				( LPARAM ) this );
}

/* static */ BOOL __stdcall c_NewSubEntry::DialogProc ( 
	/* <-- */				HWND f_hWnd, UINT f_message, WPARAM f_wParam, LPARAM f_lParam )
{
	c_NewSubEntry * pThis =
		/* <-- */ ( c_NewSubEntry * ) GetWindowLong ( f_hWnd, DWL_USER );

	switch ( f_message )
	{
	case WM_INITDIALOG:
		pThis = ( c_NewSubEntry * ) f_lParam;
		SetWindowLong ( f_hWnd, DWL_USER, ( long ) pThis );
		pThis->InitDialog ( f_hWnd );
		return true;

	case WM_COMMAND:
		switch ( LOWORD ( f_wParam ) )
		{
		case IDOK:
			pThis->SthDid ( );
			if ( ! pThis->SaveData ( ) )
				return true;
		case IDCANCEL:
			pThis->SthDid ( );
			SendMessage ( f_hWnd, WM_CLOSE, 0, 0 );
			return true;
		case IDC_HIDE_VALUE:
			SendMessage ( GetDlgItem ( pThis->m_hWnd, IDC_VALUE ), EM_SETPASSWORDCHAR,
				/* <-- */  SendMessage ( GetDlgItem ( pThis->m_hWnd, IDC_HIDE_VALUE ), BM_GETCHECK, 0, 0 ) == BST_CHECKED
				/* <-- */	  ? '*' : 0, 0 );
			SetFocus ( GetDlgItem ( pThis->m_hWnd, IDC_VALUE ) );
			SetFocus ( GetDlgItem ( pThis->m_hWnd, IDC_HIDE_VALUE ) );
			pThis->SthDid ( );
			return true;
		}
		break;

	case WM_TIMER:
		switch ( f_wParam )
		{
		case WM_TIMER_CLOSE:
			EndDialog ( f_hWnd, IDCANCEL );
			return true;
		}
		break;

	case WM_CLOSE:
		EndDialog ( f_hWnd, pThis->result );
		return true;
	}

	return false;
}


bool c_NewSubEntry::InitDialog ( HWND f_hWnd )
{
	m_hWnd = f_hWnd;

	SthDid ( );

	// Icon fr das Fenster setzen
	SendMessage ( m_hWnd, WM_SETICON, ICON_BIG,
		/* <-- */	( LPARAM ) LoadIcon ( G_hInstance, MAKEINTRESOURCE ( IDI_ICON ) ) );

	if ( m_hideValue )
	{
		SendMessage ( GetDlgItem ( m_hWnd, IDC_HIDE_VALUE ), BM_SETCHECK, BST_CHECKED, 0 );
		SendMessage ( m_hWnd, WM_COMMAND, IDC_HIDE_VALUE, 0 );
	}

	SendMessage ( GetDlgItem ( m_hWnd, IDC_ENTRY_NAME	), WM_SETTEXT, 0, ( LPARAM ) ( LPCTSTR ) m_entryName );
	SendMessage ( GetDlgItem ( m_hWnd, IDC_VALUE			), WM_SETTEXT, 0, ( LPARAM ) ( LPCTSTR ) m_value );

	return true;
}


String c_NewSubEntry::GetEntryName ( )
{
	assert ( m_wasModal );
	return m_entryName;
}

String c_NewSubEntry::GetValue ( )
{
	assert ( m_wasModal );
	return m_value;
}

bool c_NewSubEntry::GetHideValueFlag ( )
{
	assert ( m_wasModal );
	return m_hideValue;
}

bool c_NewSubEntry::SaveData ( )
{
	SthDid ( );

	m_entryName	= GetEditText ( GetDlgItem ( m_hWnd, IDC_ENTRY_NAME ) );

	if ( m_entryName.Find ( "*" ) != -1 || m_entryName.Find ( "=" ) != -1 )
	{
		MessageBox ( m_hWnd, InvalidChars, InvalidCharsTitel, MB_OK | MB_ICONEXCLAMATION );
		SthDid ( );
		return false;
	}

	m_value		= GetEditText ( GetDlgItem ( m_hWnd, IDC_VALUE ) );

	if ( m_entryName.Length ( ) == 0 || m_value.Length ( ) == 0 )
	{
		MessageBox ( m_hWnd, InvalidLength, InvalidLengthTitel, MB_OK | MB_ICONEXCLAMATION );
		SthDid ( );
		return false;
	}
	
	m_hideValue = SendMessage ( GetDlgItem ( m_hWnd, IDC_HIDE_VALUE ),
		/* <-- */					 BM_GETCHECK, 0, 0 ) == BST_CHECKED;

	result = IDOK;

	return true;
}

void c_NewSubEntry::SthDid ( )
{
	SetTimer ( m_parent, WM_TIMER_CLOSE, G_Global.timeout_newSubDialog + 1000, NULL );
	SetTimer ( m_hWnd, WM_TIMER_CLOSE, G_Global.timeout_newSubDialog, NULL );
}