/*
	SerpEncrypTool - A tool that uses GnuPG in order to manage your passwords and similar things
	Copyright (C) 2004  Michael "Serpedon" Walz

	This file is part of SerpEncrypTool.

	SerpEncrypTool is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	SerpEncrypTool is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SerpEncrypTool; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

# include <assert.h>
# include "Defines.hpp"
# include "Ressource.h"
# include "Klasse.SubDialog.hpp"
# include "Klasse.NewSubEntry.hpp"
# include "WindowFkts.hpp"

class c_SubDialogEntry
{
public:
	String name;
	String value;
	bool hide;
	String CreateEntry ( )
	{
		String result;
		if ( hide )
			result.Format ( SubDialogEntryHideFormat, name, value );
		else
			result.Format ( SubDialogEntryFormat, name, value );
		return result;
	}
	String CreateDataFromEntry ( )
	{
		String result;
		result.Format ( "%s%s=%s\n", name, hide ? "*" : "" , value );
		return result;
	}
	bool Init ( const String & f_data )
	{
		int pos = f_data.Find ( "=" );
		
		// Erste Stelle wrde heien, kein Name vorhanden
		// -1 heit kein = drin
		if ( pos <= 0 )
			return false;

		if ( f_data [ pos - 1 ] == '*' )
			hide = true;
		else
			hide = false;

		name = f_data.Mid ( 0, pos - ( hide ? 1 : 0 ) );

		value = f_data.Mid ( pos + 1 );

		if ( name.Length ( ) == 0 || value.Length ( ) == 0 )
			return false;

		return true;
	}
};

c_SubDialog::c_SubDialog ( ) :
	m_parent ( NULL ), m_hWnd ( NULL ), m_clearClipboardOnEnd ( false )
{
	# if _DEBUG
		m_isInit = false;
		m_wasModal = false;
	# endif
}

c_SubDialog::~c_SubDialog ( )
{
}


bool c_SubDialog::Init ( HWND f_parent, const String & f_data )
{
	# if _DEBUG
		assert ( ! m_isInit );
		m_isInit = true;
	# endif
	
	m_parent	= f_parent;
	m_data	= f_data;

	if ( IsWindow ( m_parent ) )
		return true;

	return false;
}

int c_SubDialog::DoModal ( )
{
	# if _DEBUG
		assert ( m_isInit );
		assert ( ! m_wasModal );
		m_wasModal = true;
	# endif
	return DialogBoxParam ( G_hInstance,
		/* <-- */				MAKEINTRESOURCE ( IDD_SUB_DIALOG ),
		/* <-- */				m_parent,
		/* <-- */				DialogProc,
		/* <-- */				( LPARAM ) this );
}

/* static */ BOOL __stdcall c_SubDialog::DialogProc ( 
	/* <-- */				HWND f_hWnd, UINT f_message, WPARAM f_wParam, LPARAM f_lParam )
{
	c_SubDialog * pThis =
		/* <-- */ ( c_SubDialog * ) GetWindowLong ( f_hWnd, DWL_USER );

	switch ( f_message )
	{
	case WM_INITDIALOG:
		pThis = ( c_SubDialog * ) f_lParam;
		SetWindowLong ( f_hWnd, DWL_USER, ( long ) pThis );
		if ( ! pThis->InitDialog ( f_hWnd ) )
			SendMessage ( f_hWnd, WM_CLOSE, 0, 0 );
		return true;

	case WM_COMMAND:
		switch ( LOWORD ( f_wParam ) )
		{
		case IDOK:
			if ( G_Global.disableEnter )
				break;
		case ID_SHOW_ENTRY:
			pThis->ShowEntry ( );
			return true;
		case IDCANCEL:
			SendMessage ( f_hWnd, WM_CLOSE, 0, 0 );
			return true;
		case ID_COPY_CLIPBOARD:
			pThis->CopyToClipboard ( );
			return true;
		case ID_ADD_ENTRY:
			pThis->AddEntry ( );
			return true;
		case ID_DELETE_ENTRY:
			pThis->DeleteEntry ( );
			return true;
		case ID_EDIT_ENTRY:
			pThis->EditEntry ( );
			return true;
		case IDC_LIST_DATA:
			if ( HIWORD ( f_wParam ) == LBN_DBLCLK )
			{
				if ( ! G_Global.disableDoubleClick )
					pThis->ShowEntry ( );
				return true;
			}
			break;
		}
		break;
	case WM_TIMER:
		switch ( f_wParam )
		{
		case WM_TIMER_CLOSE:
			SendMessage ( f_hWnd, WM_CLOSE, 0, 0 );
			return true;
		}
		break;
	case WM_CLOSE:
		return pThis->CloseDialog ( );
	}

	return false;
}


bool c_SubDialog::InitDialog ( HWND f_hWnd )
{
	m_hWnd = f_hWnd;

	// Icon fr das Fenster setzen
	SendMessage ( m_hWnd, WM_SETICON, ICON_BIG,
		/* <-- */	( LPARAM ) LoadIcon ( G_hInstance, MAKEINTRESOURCE ( IDI_ICON ) ) );

	SthDid ( );

	String temp = m_data;

	const char * seperator = "\n";
	char * token = strtok ( temp, seperator );
   while ( token != NULL )
   {
		c_SubDialogEntry * p_newEntry = new c_SubDialogEntry ( );
		if ( p_newEntry == NULL )
			return false;

		if ( ! p_newEntry->Init ( token ) )
		{
			MessageBox ( m_hWnd, InvalidToken, InvalidTokenTitel, MB_OK | MB_ICONEXCLAMATION );
			delete p_newEntry;
			return false;
		}

		AddListBoxItemAndLink ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ), p_newEntry->CreateEntry ( ), p_newEntry );

      token = strtok ( NULL, seperator );
   }

	return true;
}

bool c_SubDialog::CloseDialog ( )
{
	if ( m_clearClipboardOnEnd )
	{
		if ( OpenClipboard ( NULL ) && EmptyClipboard ( ) )
			CloseClipboard ( );
		else
			MessageBox ( m_hWnd, ClipboardNotDeleted, ClipboardNotDeletedTitel, MB_OK | MB_ICONEXCLAMATION );
	}

	// Neue Daten aus dem Listenobjekt auslesen
	int amount = SendMessage ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ), LB_GETCOUNT, 0, 0 );
	if ( amount != LB_ERR )
	{
		m_data = "";
		for ( int count = 0; count < amount; count++ )
		{
			c_SubDialogEntry * p_entry = ( c_SubDialogEntry * )
				/* <-- */							SendMessage ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ),
				/* <-- */							LB_GETITEMDATA, count, 0 );

			m_data += p_entry->CreateDataFromEntry ( );
			
			delete p_entry;
		}
	}
	else
		MessageBox ( m_hWnd, ErrorWhileSavingEntries, SavingFailed, MB_OK | MB_ICONEXCLAMATION );

	EndDialog ( m_hWnd, 0 );
	return true;
}

String c_SubDialog::GetNewData ( )
{
	assert ( m_wasModal );
	return m_data;
}


bool c_SubDialog::ShowEntry ( )
{
	c_SubDialogEntry * p_entry = ( c_SubDialogEntry * ) GetCurrentSelection ( NULL );

	SthDid ( );

	if ( p_entry )
	{
		if ( p_entry->hide )
		{
			if ( IDYES != MessageBox ( m_hWnd, WillBeShownAsPlain, ShowAsPlain, MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2 ) )
				return true;
		}

		MessageBox ( m_hWnd, p_entry->value, p_entry->name, MB_OK | MB_ICONINFORMATION );
	}
	else
		MessageBox ( m_hWnd, NoEntrySelected, NoEntrySelectedTitel, MB_OK | MB_ICONEXCLAMATION );

	SthDid ( );

	return true;
}

bool c_SubDialog::CopyToClipboard ( )
{
	SthDid ( );
	
	c_SubDialogEntry * p_entry = ( c_SubDialogEntry * ) GetCurrentSelection ( NULL );

	if ( p_entry )
	{
		bool result = false;
		if ( OpenClipboard ( m_hWnd ) )
		{
			if ( EmptyClipboard ( ) )
			{
				HGLOBAL hData;
				LPVOID pData;
				hData = GlobalAlloc ( GMEM_DDESHARE | GMEM_MOVEABLE, strlen ( p_entry->value ) + 1 );
				if ( hData )
				{
					if ( pData = GlobalLock ( hData ) )
					{
						strcpy ( ( char * ) pData, p_entry->value );
						GlobalUnlock ( hData );
						if ( SetClipboardData ( CF_TEXT, hData ) )
						{
							m_clearClipboardOnEnd = true;
							result = true;
						}
					}
				}
			}

			CloseClipboard ( );
		}

		if ( ! result )
			MessageBox ( m_hWnd, CouldNotSetClipboardData, ClipboardError, MB_OK | MB_ICONEXCLAMATION );

		SthDid ( );
		
		return result;
	}
	else
		MessageBox ( m_hWnd, NoEntrySelected, NoEntrySelectedTitel, MB_OK | MB_ICONEXCLAMATION );

	SthDid ( );

	return true;
}

bool c_SubDialog::AddEntry ( )
{
	SthDid ( );

	bool result = false;
	c_NewSubEntry newEntry;
	if ( newEntry.Init ( m_hWnd ) )
	{
		if ( newEntry.DoModal ( ) == IDOK )
		{
			c_SubDialogEntry * p_newEntry = new c_SubDialogEntry ( );
			if ( p_newEntry )
			{
				p_newEntry->value	= newEntry.GetValue ( );
				p_newEntry->name	= newEntry.GetEntryName ( );
				p_newEntry->hide	= newEntry.GetHideValueFlag ( );				

				if ( AddListBoxItemAndLink ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ),
					/* <-- */		p_newEntry->CreateEntry ( ), p_newEntry ) != LB_ERR )
					result = true;
			}
		}
	}

	return result;
}

bool c_SubDialog::EditEntry ( )
{
	SthDid ( );

	int index;
	c_SubDialogEntry * p_entry = ( c_SubDialogEntry * ) GetCurrentSelection ( & index );

	bool result = false;
	
	if ( p_entry )
	{
		c_NewSubEntry newEntry;
		if ( newEntry.Init ( m_hWnd, p_entry->name, p_entry->value, p_entry->hide ) )
		{
			if ( newEntry.DoModal ( ) == IDOK )
			{
				SendMessage ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ), LB_DELETESTRING, index, 0 );

				p_entry->value	= newEntry.GetValue ( );
				p_entry->name	= newEntry.GetEntryName ( );
				p_entry->hide	= newEntry.GetHideValueFlag ( );				

				if ( AddListBoxItemAndLink ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ),
					/* <-- */		p_entry->CreateEntry ( ), p_entry ) != LB_ERR )
					result = true;
			}
		}
	}
	else
		MessageBox ( m_hWnd, NoEntrySelected, NoEntrySelectedTitel, MB_OK | MB_ICONEXCLAMATION );

	return result;
}


bool c_SubDialog::DeleteEntry ( )
{
	SthDid ( );

	int index;
	c_SubDialogEntry * p_entry = ( c_SubDialogEntry * ) GetCurrentSelection ( & index );

	if ( p_entry )
	{
		if ( IDYES == MessageBox ( m_hWnd, ReallyDelete, RealleyDeleteTitel,
			/* <-- */					MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2 ) )
		{
			SendMessage ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ), LB_DELETESTRING, index, 0 );
			delete p_entry;
		}
	}
	else
		MessageBox ( m_hWnd, NoEntrySelected, NoEntrySelectedTitel, MB_OK | MB_ICONEXCLAMATION );

	SthDid ( );

	return true;
}

void * c_SubDialog::GetCurrentSelection ( int * fp_curSel )
{
	int temp;
	if ( fp_curSel == NULL )
		fp_curSel = & temp;

	( * fp_curSel ) = SendMessage ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ), 
		/* <-- */						  LB_GETCURSEL, ( * fp_curSel ), 0 ); 

	void * result = ( void * ) SendMessage ( GetDlgItem ( m_hWnd, IDC_LIST_DATA ), 
		/* <-- */									  LB_GETITEMDATA, ( * fp_curSel ), 0 );

	if ( result == ( void * ) LB_ERR )
		return NULL;

	return result;
}

void c_SubDialog::SthDid ( )
{
	SetTimer ( m_hWnd, WM_TIMER_CLOSE, G_Global.timeout_subDialog, NULL );
}